/*
 * @(#)$Id: $
 *
 * Copyright 2002 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package org.kohsuke.rmeter;

import java.text.MessageFormat;
import java.util.HashSet;
import java.util.Set;

import com.sun.msv.grammar.AttributeExp;
import com.sun.msv.grammar.ElementExp;
import com.sun.msv.grammar.Expression;
import com.sun.msv.grammar.NameClass;
import com.sun.msv.grammar.NameClassAndExpression;
import com.sun.msv.grammar.util.ExpressionWalker;
import com.sun.msv.grammar.util.NameClassCollisionChecker;

/**
 * 
 * @author
 * 	Kohsuke Kawaguchi (kohsuke.kawaguchi@sun.com)
 */
public class SingleTypeTester {
    
    public SingleTypeTester( CollisionHandler _handler ) {
        this.handler = _handler;
    }
    
    private CollisionHandler handler;
    
    /** already tested expressions. */
    private final Set testedExps = new HashSet();
    
    public void test( ElementExp owner, Expression exp ) {
        if(!testedExps.add(exp))
            return; // already tested
        
        // build a set of attributes/elements and
        // make sure none of them corride.
        
        final Set elements = new HashSet();
        final Set attributes = new HashSet();
        exp.visit(new ExpressionWalker(){
            public void onElement( ElementExp exp ) {
                elements.add(exp);
            }
            public void onAttribute( AttributeExp exp ) {
                attributes.add(exp);
            }
        });
        
/*        System.out.println(MessageFormat.format(
            "tested {0} elements and {1} attributes",
            new Object[]{
                new Integer(elements.size()),
                new Integer(attributes.size())}));
*/
        
        ElementExp[] es = (ElementExp[])
            elements.toArray(new ElementExp[elements.size()]);
        collisionTest(owner,es);
        collisionTest(owner, (AttributeExp[])
            attributes.toArray(new AttributeExp[attributes.size()]));
        
        // recursively test children
        for( int i=0; i<es.length; i++ )
            test(es[i],es[i].contentModel);
    }
    
    
    private void collisionTest( ElementExp owner, NameClassAndExpression[] exps ) {
        for( int i=0; i<exps.length; i++ )
            for( int j=i+1; j<exps.length; j++ )
                test(owner,exps[i],exps[j]);
    }
    
    private final NameClassCollisionChecker checker = new NameClassCollisionChecker();
    
    private void test( ElementExp owner,
        NameClassAndExpression exp1, NameClassAndExpression exp2 ) {
        
        if(checker.check(exp1.getNameClass(),exp2.getNameClass()))
            handler.onCollision(owner,exp1,exp2);
    }
}
